package nova.monadic_sfx

import scalafx.application.JFXApp
import scalafx.application.JFXApp.PrimaryStage
import monix.eval.Task
import monix.execution.Scheduler
// import sttp.client.asynchttpclient.monix.AsyncHttpClientMonixBackend
// import sttp.client._
// import sttp.client.circe._
// import io.circe.generic.auto._
import scala.util.Failure
import scala.util.Success
import akka.{actor => classic}
import nova.monadic_sfx.executors._
import cats.effect.Resource
import nova.monadic_sfx.models._
import nova.monadic_sfx.ui.DefaultUI
import nova.monadic_sfx.http.Backend
import nova.monadic_sfx.modules.MainModule
import scalafx.stage.Stage
import scalafx.scene.layout.FlowPane
import nova.monadic_sfx.pages.LoginPage
import scala.concurrent.duration._
import sttp.client.asynchttpclient.monix.AsyncHttpClientMonixBackend
import scalafx.Includes._
import scala.concurrent.Await
import akka.actor.typed.scaladsl.adapter._
import scalafx.application.Platform
import io.odin.syntax._
// import io.odin._
import io.odin.monix._
import nova.monadic_sfx.pages.HomePage
import monix.execution.Callback
import monix.eval.TaskApp
import cats.effect.ExitCode
import scalafx.scene.layout.HBox
import scalafx.scene.Scene
import cats.implicits._
import cats.effect.Clock
import cats.effect.Sync

object Main extends TaskApp {

  // lazy val schedulers: Schedulers = new Schedulers()
  // override implicit def scheduler: Scheduler = schedulers.fx
  override def run(args: List[String]): Task[ExitCode] = {
    // val logger = consoleLogger().withAsyncUnsafe()

    // lazy val backendTask = AsyncHttpClientMonixBackend()
    // lazy val actorSystemTask = Task {
    //   classic.ActorSystem(
    //     name = "FXActorSystem"
    //   )
    // }

    // implicit lazy val FSync = Sync[Task]
    // implicit lazy val FClock = Clock[Task]
    // val startTime = Task.clock
    //   .monotonic(scala.concurrent.duration.MILLISECONDS)
    //   .map(Duration.fromNanos(_))
    lazy val appResource = for {
      // clock <- Resource.liftF(Task(Task.clock))
      logger <- consoleLogger().withAsync()
      backend <- AsyncHttpClientMonixBackend.resource()
      actorSystem <-
        Resource.make(logger.info("Creating Actor System") >> Task {
          classic.ActorSystem(
            name = "FXActorSystem"
          )
        })(sys =>
          logger.info("Shutting down actor system") >> Task.fromFuture(
            sys.terminate()
          ) >> logger.info("Actor System terminated")
        )
      // _ <- Resource.liftF(logger.info(Thread.currentThread().getName()))
      fxApp <- Resource.make(logger.info("Creating FX Application") >> Task {
        // val appStage = makePrimaryStage(backend, actorSystem)
        // stage = appStage
        // val stage2 = new PrimaryStage {
        //   scene = new Scene(new HBox())
        // }
        val app: JFXApp = new JFXApp {
          lazy val schedulers: Schedulers = new Schedulers()

          implicit lazy val defaultScheduler: Scheduler = schedulers.fx

          val application =
            for {
              appStage <- logger.info("Inside JFX application stage") >> Task(
                makePrimaryStage(backend, actorSystem)
              )
              _ <- Task { stage = appStage }
              _ <- Task.sleep(2.seconds)
              loginScene <- LoginPage(appStage, backend, actorSystem)
              _ <- Task {
                // appStage.maximized = true
                appStage.height = 800
                appStage.width = 800
                appStage
                  .scene()
                  .setRoot(
                    loginScene
                  )
              }
            } yield ()
          application.timed.runAsync(
            new Callback[Throwable, (FiniteDuration, Unit)] {

              override def onSuccess(value: (FiniteDuration, Unit)): Unit = {
                val (duration, _) = value
                println(
                  s"Application started successfully in ${duration.toSeconds} seconds"
                )
              }

              override def onError(e: Throwable): Unit = {
                println("Application start failed. Reason -")
                e.printStackTrace()
              }

            }
          )

          override def stopApp() = {
            Platform.exit()
            // System.exit(0)
          }
        }
        app
      })(app => logger.info("Stopping FX Application") >> Task(app.stopApp()))
      // _ <- Resource.liftF(Task.unit.executeOn(defaultScheduler))
      _ <- Resource.liftF(logger.info(Thread.currentThread().getName()))
    } yield (fxApp)
// >> logger.info("test")
    appResource
      .use(fxApp => Task(fxApp.main(args.toArray)))
      .as(ExitCode.Success)
  }

  def makePrimaryStage(
      backend: AppTypes.HttpBackend,
      actorSystem: classic.ActorSystem
  ) = {
    new PrimaryStage {
      scene = new DefaultUI().scene
      // onCloseRequest = () => {
      // val f2 = actorSystem.terminate()
      // val f1 = backend.close().runToFuture

      // println("Closing backend")
      // Await.result(f1, 3.seconds)
      // println("Closing actor system")
      // println(Thread.currentThread().getName())
      // Await.result(f2, 3.seconds)
      // println("Actor system closed")
      // Platform.exit()
      // System.exit(0)
      // }
    }
  }
}

// class MyFxApp extends javafx.application.Application {

//   override def start(stage: javafx.stage.Stage): Unit = {
//     stage.show()
//   }

// }
